/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef AUTOMATICINTERRUPTLOCK_H
#define AUTOMATICINTERRUPTLOCK_H

#include <stdint.h>

#include "globalinterruptflag.h"

/**
 * Class that implements a lock mechanism by disabling the global interrupt flag.
 * The previous state of SREG is stored and restored, so this lock can also be used if the interrupts are
 * already disabled without accidentally enabling them.
 * @note When the instance is created the global interrupt flag is cleared, when the instance is destroyed the
 *       flag is restored. The idea is to use this to lock an entire block scope. When you need to protect a
 *       fraction inside a block, consider using GlobalInterruptFlag::clear and GlobalInterruptFlag::restore
 *       manually.
 */
class AutomaticInterruptLock
{
public:
	/**
	 * Constructor, automatically shuts the lock by disabling the global interrupt flag.
	 */
	AutomaticInterruptLock(void)
	{
		m_sreg = GlobalInterruptFlag::clear();
	}

	/**
	 * Destructor, automatically opens the lock by restoring the previous state of the global interrupt flag.
	 */
	~AutomaticInterruptLock(void)
	{
		GlobalInterruptFlag::restore(m_sreg);
	}

private:
	uint8_t m_sreg;
};

#endif
