/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef BUFFEREDUSART_H
#define BUFFEREDUSART_H

#include "simpleusart.h"
#include "synchronizedringbuffer.h"

/**
 * This template class implements an USART with separate buffers for incoming and outgoing data.
 * The class makes use of the USART's RX Complete and Data Register Empty interrupts.
 * @param RXQUEUESIZE Size of the queue for incoming data in bytes
 * @param TXQUEUESIZE Size of the queue for outgoing data in bytes
 */
template <uint8_t RXQUEUESIZE, uint8_t TXQUEUESIZE>
class BufferedUsart : public SimpleUsart, public Usart::InterruptHandler
{
public:
	/**
	 * Constructor.
	 * @param usart Pointer to USART device to be used.
	 */
	BufferedUsart(Usart *usart) : SimpleUsart(usart)
	{
		if (m_usart)
		{
			m_usart->setInterruptHandler(this);
		}
	}

	/**
	 * Enables or disables the USART on hardware level.
	 * This also enables or disables the RX Complete Interrupt.
	 * @param state Desired state of USART, true means enabled, false means disabled.
	 */
	void enable(bool state)
	{
		SimpleUsart::enable(state);

		if (m_usart)
		{
			// Enable/disable Rx interrupt only; tx must be kept for making sure the queue gets emptied.
			m_usart->enableRxCompleteInterrupt(state);
		}
	}

	/**
	 * Gets a byte from the incoming data buffer, if available.
	 * @param byte Pointer to buffer that receives the data byte.
	 * @result True if a data byte was read, false otherwise.
	 */
	bool read(uint8_t *byte)
	{
		bool result = false;

		if (m_usart && byte)
		{
			if (m_rxQueue.dequeue(byte))
			{
				result = true;
			}
		}

		return result;
	}

	/**
	 * Puts a byte in the outgoing data buffer, if possible. This also enables the Data Register
	 * Empty interrupt to continue with emptying the buffer.
	 * @param byte Data byte to send.
	 * @result True if data was enqueued, false otherwise.
	 */
	bool write(uint8_t byte)
	{
		bool result = false;

		if (m_usart)
		{
			if (m_txQueue.enqueue(byte))
			{
				m_usart->enableDataRegisterEmptyInterrupt(true);
				result = true;
			}
		}

		return result;
	}

	/**
	 * Handler for Data Register Empty interrupt.
	 * Used to put next data byte from outgoing data buffer into data register.
	 * When no data is left the Data Register Empty interrupt is disabled.
	 * @note This method is called from ISR context. Don't perform lengthy operations here.
	 */
	virtual void onDataRegisterEmpty(void) override
	{
		uint8_t byte;

		if (m_txQueue.dequeue(&byte))
		{
			m_usart->setData(byte);
		}
		else
		{
			m_usart->enableDataRegisterEmptyInterrupt(false);
		}
	}

	/**
	 * Handler for RX Complete interrupt. Called in ISR context.
	 * Used to get data byte from data register and to store byte in incoming
	 * data buffer. If buffer is full the date byte is discarded.
	 * @note This method is called from ISR context. Don't perform lengthy operations here.
	 */
	virtual void onRxComplete(void) override
	{
		uint8_t byte = m_usart->getData();
		m_rxQueue.enqueue(byte);
	}

private:
	SynchronizedRingBuffer<uint8_t, RXQUEUESIZE> m_rxQueue;
	SynchronizedRingBuffer<uint8_t, TXQUEUESIZE> m_txQueue;
};

#endif
