/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef EVENTDISPATCHER_H
#define EVENTDISPATCHER_H

#include <stdint.h>

#include "event.h"
#include "eventreceiver.h"
#include "automaticinterruptlock.h"
#include "itemcollection.h"

/**
 * This component is able to receive events and to forward them to a set of subscribers.
 * The events are forwarded in the same context as they are received.
 * @param SUBSCRIBERCOUNT Maximum number of subscribers.
 */
template <uint8_t SUBSCRIBERCOUNT>
class EventDispatcher : public EventReceiver
{
public:
	/**
	 * Subscribe to event dispatcher. Fails if already subscribed.
	 * @param subscriber Pointer to EventReceiver to add to the subscriber list.
	 * @result True if adding subscription was successful, false otherwise.
	 */
	bool subscribe(EventReceiver *subscriber)
	{
		AutomaticInterruptLock lock;
		return m_subscribers.addUnique(subscriber);
	}

	/**
	 * Remove existing subscription.
	 * @param subscriber Pointer to EventReceiver to remove from subscriber list.
	 */
	void unsubscribe(EventReceiver *subscriber)
	{
		AutomaticInterruptLock lock;
		m_subscribers.remove(subscriber);
	}

	/**
	 * Dispatches event to all subscribers.
	 * @param event Event to dispatch.
	 */
	virtual void onEvent(const Event &event) override
	{
		// See class description of ItemCollection to understand why we don't use a capture here
		auto cb = [](EventReceiver *&r, const void *ctx){r->onEvent(*static_cast<const Event*>(ctx));};
		m_subscribers.forEach(cb, &event);
	}

private:
	ItemCollection<EventReceiver*, SUBSCRIBERCOUNT> m_subscribers;
};

#endif
