/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef EXTERNALINTERRUPT_H
#define EXTERNALINTERRUPT_H

#include <stdint.h>

/**
 * Class to manage the external interrupt sources provided by the AVR (INT0, INT1, etc.).
 * @note Managing the interrupt sources is independent from controlling the associated GPIO pin.
 *       To control the actual pin, a separate instance of GpioPortPin is needed. It can be configured
 *       as input to react on external circuitry, or as output to trigger interrupts from software.
 */
class ExternalInterrupt
{
public:
	/**
	 * Callback type for use with interrupts.
	 */
	class InterruptHandler
	{
	public:
		/**
		 * Handler for external interrupts.
		 * @todo Does this need to carry the information *which* external interrupts
		 *       occurred, in case of using a single object to handle multiple sources?
		 */
		virtual void onExternalInterrupt(void) = 0;
	};

	/**
	 * Trigger modes.
	 */
	enum TriggerMode
	{
		/// Pin is held at low level.
		LOW_LEVEL,
		/// Pin changes its level.
		ANY_CHANGE,
		/// Pin changes from high to low.
		FALLING_EDGE,
		/// Pin changes from low to high.
		RISING_EDGE,
	};

	/**
	 * Sets the trigger mode.
	 * @param mode Trigger mode to set.
	 */
	virtual void setTriggerMode(TriggerMode mode) = 0;

	/**
	 * Enables or disabled the interrupt.
	 * @param state True to enable interrupt, false to disable.
	 */
	virtual void enableInterrupt(bool state) = 0;

	/**
	* Sets callback for interrupts.
	* @param handler Pointer to class implementing the interrupt handler interface.
	*/
	virtual void setInterruptHandler(InterruptHandler *handler) = 0;
};

#endif
