/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef NOTIFYINGBUFFEREDUSART_H
#define NOTIFYINGBUFFEREDUSART_H

#include "eventreceiver.h"
#include "bufferedusart.h"

/**
 * Extension of the BufferedUsart that sends notifications using the event system when data becomes ready
 * for reading or when there is new capacaity for writing data.
 * The notifications are sent only once the trigger occurs, until the user re-enables them by reading data (for the
 * readable notification) or writing data (for the writable notification).
 * @note Keep in mind that sending of events happens in ISR context. When using an EventDispatcher with autoDispatch
 *       set to true as receiver, all its subscribers will also be notified within ISR context!
 */
template <uint8_t RXQUEUESIZE, uint8_t TXQUEUESIZE>
class NotifyingBufferedUsart : public BufferedUsart<RXQUEUESIZE, TXQUEUESIZE>
{
public:
	/**
	 * Constructor.
	 * @param usart Pointer to USART device to be used.
	 * @param receiver Pointer to EventReceiver instance to receive notifications.
	 * @param readableEvent Event to be sent when data was added to the incoming buffer (device is readable).
	 * @param writableEvent Event to be sent when data was removed from the outgoing buffer (device is writable).
	 * @note You can pass INVALID_EVENT_ID for events that should not be sent.
	 */
	NotifyingBufferedUsart(Usart *usart, EventReceiver *receiver, const Event &readableEvent, const Event &writableEvent) :
		BufferedUsart<RXQUEUESIZE, TXQUEUESIZE>(usart), m_receiver(receiver), m_readableEvent(readableEvent),
		m_writableEvent(writableEvent), m_readableNotified(false), m_writableNotified(false)
	{
	}

	/**
	 * Re-enables readable notification and gets a byte from the buffer, if available.
	 * @param byte Pointer to buffer that receives the data byte.
	 * @result True if a data byte was read, false otherwise.
	 */
	bool read(uint8_t *byte)
	{
		m_readableNotified = false;
		return BufferedUsart<RXQUEUESIZE, TXQUEUESIZE>::read(byte);
	}

	/**
	 * Re-enables writable notification and puts a byte into the buffer, if possible.
	 * @param byte Data byte to send.
	 * @result True if data was enqueued, false otherwise.
	 */
	bool write(uint8_t byte)
	{
		m_writableNotified = false;
		return BufferedUsart<RXQUEUESIZE, TXQUEUESIZE>::write(byte);
	}

	/**
	 * Calls base class implementation and sends writeable notification, if not already done.
	 */
	virtual void onDataRegisterEmpty(void) override
	{
		BufferedUsart<RXQUEUESIZE, TXQUEUESIZE>::onDataRegisterEmpty();
		if (!m_writableNotified)
		{
			if (m_receiver && m_writableEvent.getId() != Event::INVALID_EVENT_ID)
			{
				m_receiver->onEvent(m_writableEvent);
				m_writableNotified = true;
			}
		}
	}

	/**
	 * Calls base class implementation and sends readable notification, if not already done.
	 */
	virtual void onRxComplete(void) override
	{
		BufferedUsart<RXQUEUESIZE, TXQUEUESIZE>::onRxComplete();
		if (!m_readableNotified)
		{
			if (m_receiver && m_readableEvent.getId() != Event::INVALID_EVENT_ID)
			{
				m_receiver->onEvent(m_readableEvent);
				m_readableNotified = true;
			}
		}
	}

private:
	EventReceiver * const m_receiver;
	Event m_readableEvent;
	Event m_writableEvent;
	bool m_readableNotified;
	bool m_writableNotified;
};

#endif
