/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef PINARRAY_H
#define PINARRAY_H

/**
 * Template class that allows to build an array of pins with arbitrary type. The use-case is to form a virtual port
 * consisting of pins that can be located anywhere. Although PinArray features both, a getValue() and a setValue()
 * method, depending on the pin type being used only one or the other might be actually available.
 * Using DATATYPE and WIDTH it is possible to even form virtual ports of sizes greater than 8 bit.
 * For convenient access the classes VirtualInputPort and VirtualOutputPort exist, which implement 8 bit ports.
 * @param PINTYPE Type of pin to use, e.g. InputPin. Type must be provided by user, e.g. by proper \#include.
 * @param DATATYPE Data type internally used for bit shifting and as parameters, e.g. uint8_t.
 * @param WIDTH Width of virtual port, may be smaller than DATATYPE supports, e.g. 6 bit.
 * @note Be warned: just because this template compiles, it doesn't need to make sense. You can create a virtual port
 *       with 32 or even more bits, but using it will be a pain for an 8 bit CPU...
 * @note You can mix real pins with null pointers, or even reference the same pins multiple times.
 */
template <class PINTYPE, typename DATATYPE, uint8_t WIDTH>
class PinArray
{
public:
	/**
	 * Constructor, sets all pins to null pointers.
	 */
	PinArray()
	{
		for (uint8_t i = 0; i < WIDTH; i++)
		{
			m_pins[i] = nullptr;
		}
	}

	/**
	 * Sets the pin to be used for the specified bit position.
	 * @param index Bit position to set pin for.
	 * @param pin Pin to use for this bit position.
	 */
	void setPin(uint8_t index, PINTYPE *pin)
	{
		if (index < WIDTH)
		{
			m_pins[index] = pin;
		}
	}

	/**
	 * Get value being represented by set of pins.
	 * @result Value read from pins.
	 * @note Supported only if underlying PINTYPE supports getValue().
	 * @note Bits that are not associated with a pin read as 0.
	 */
	DATATYPE getValue(void) const
	{
		DATATYPE value = 0;
		DATATYPE currMask = 1u;

		for (uint8_t i = 0; i < WIDTH; i++)
		{
			if (m_pins[i])
			{
				value |= m_pins[i]->getValue() ? currMask : 0;
			}
			currMask = currMask << 1;
		}

		return value;
	}

	/**
	 * Set value being represented by set of pins.
	 * @param value Value to be set.
	 * @param mask Mask with pins to manipulate according to value.
	 * @note Supported only if underlying PINTYPE supports setValue().
	 */
	void setValue(DATATYPE value, DATATYPE mask)
	{
		DATATYPE currMask = 1u;

		for (uint8_t i = 0; i < WIDTH; i++)
		{
			if (currMask & mask)
			{
				if (m_pins[i])
				{
					m_pins[i]->setValue((value & currMask) != 0);
				}
			}
			currMask = currMask << 1;
		}
	}

private:
	PINTYPE *m_pins[WIDTH];
};

#endif
