/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef RINGBUFFER_H
#define RINGBUFFER_H

/**
 * Template class that implements a ring buffer with elements of arbitrary type.
 * @note This buffer must be used from a single context only. If you are looking for a thread-safe
 *       buffer, please use SynchronizedRingBuffer.
 * @param BASETYPE Type of elements in ring buffer
 * @param COUNT Maximum count of elements that can be placed in ring buffer
 */
template <class BASETYPE, uint16_t COUNT> class RingBuffer
{
public:
	RingBuffer(void) : m_readPos(0), m_writePos(0), m_empty(true), m_full(false)
	{
	}

	/**
	 * Adds an element into the ringer buffer.
	 * @param item Item to add.
	 * @result True if element could be added, false otherwise.
	 */
	bool enqueue(BASETYPE item)
	{
		bool result = false;

		if (!m_full)
		{
			m_buffer[m_writePos] = item;
			if (++m_writePos >= COUNT)
			m_writePos = 0;
			m_empty = false;
			m_full = m_writePos == m_readPos;
			result = true;
		}

		return result;
	}

	/**
	 * Gets an element from the ring buffer.
	 * @param item Pointer to buffer to receive the element.
	 * @result True if element could be retrieved, false otherwise.
	 */
	bool dequeue(BASETYPE *item)
	{
		bool result = false;

		if (!m_empty)
		{
			*item = m_buffer[m_readPos];
			if (++m_readPos >= COUNT)
			m_readPos = 0;
			m_empty = m_readPos == m_writePos;
			m_full = false;
			result = true;
		}

		return result;
	}

	/**
	 * Returns if buffer is empty.
	 * @result True if buffer is empty, false otherwise.
	 */
	bool isEmpty(void)
	{
		return m_empty;
	}

	/**
	 * Returns if buffer is full.
	 * @result True if buffer is full, false otherwise.
	 */
	bool isFull(void)
	{
		return m_full;
	}

private:
	BASETYPE m_buffer[COUNT];
	uint16_t m_readPos;
	uint16_t m_writePos;
	bool m_empty;
	bool m_full;
};

#endif
