/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef STOPWATCHTIMER_H
#define STOPWATCHTIMER_H

#include "automaticinterruptlock.h"
#include "timerservice.h"

/**
 * A StopWatchTimer is an upcounting timer that can be started, stopped and reset to zero. The current
 * timer value can be queried at any time. The value silently flows over when its maximum is reached.
 * The StopWatchTimer needs to be added to a TimerServiceHost instance. The base tick frequency of the
 * TimerServiceHost specifies the rate at which the timer value is incremented.
 * @param COUNTERTYPE Data type for timeout value, e.g. uint16_t.
 */
template <class COUNTERTYPE>
class StopWatchTimer : public TimerService
{
public:
	/**
	 * Constructor.
	 */
	StopWatchTimer(void) : m_value(0)
	{
	}

	virtual void onTick(void) override
	{
		if (isRunning())
		{
			m_value++;
		}
	}

	/**
	 * Resets the timeout value to zero.
	 */
	void reset(void)
	{
		AutomaticInterruptLock lock;
		m_value = 0;
	}

	/**
	 * Gets the current timer value.
	 * @result Current timer value
	 */
	COUNTERTYPE getValue(void) const
	{
		AutomaticInterruptLock lock;
		return m_value;
	}

private:
	COUNTERTYPE m_value;
};

#endif
