/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef SYNCHRONIZEDRINGBUFFER_H
#define SYNCHRONIZEDRINGBUFFER_H

#include "ringbuffer.h"
#include "automaticinterruptlock.h"

/**
 * Template class that extends the RingBuffer by locking interrupts globally for each action.
 * @param BASETYPE Type of elements in ring buffer
 * @param COUNT Maximum count of elements that can be placed in ring buffer
 */
template <class BASETYPE, uint16_t COUNT> class SynchronizedRingBuffer : public RingBuffer<BASETYPE, COUNT>
{
public:
	/**
	 * Adds an element into the ringer buffer.
	 * @param item Item to add.
	 * @result True if element could be added, false otherwise.
	 */
	bool enqueue(BASETYPE item)
	{
		AutomaticInterruptLock lock;

		bool result = RingBuffer<BASETYPE, COUNT>::enqueue(item);
		return result;
	}

	/**
	 * Gets an element from the ring buffer.
	 * @param item Pointer to buffer to receive the element.
	 * @result True if element could be retrieved, false otherwise.
	 */
	bool dequeue(BASETYPE *item)
	{
		AutomaticInterruptLock lock;

		bool result = RingBuffer<BASETYPE, COUNT>::dequeue(item);
		return result;
	}

	/**
	 * Returns if buffer is empty.
	 * @result True if buffer is empty, false otherwise.
	 */
	bool isEmpty(void)
	{
		AutomaticInterruptLock lock;

		bool result = RingBuffer<BASETYPE, COUNT>::isEmpty();
		return result;
	}

	/**
	 * Returns if buffer is full.
	 * @result True if buffer is full, false otherwise.
	 */
	bool isFull(void)
	{
		AutomaticInterruptLock lock;

		bool result = RingBuffer<BASETYPE, COUNT>::isFull();
		return result;
	}
};

#endif
