/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef TIMERADAPTER_H
#define TIMERADAPTER_H

#include "timer.h"
#include "event.h"
#include "eventreceiver.h"

/**
 * This component basically converts timer interrupts into events. For each interrupt a separate event ID can be set.
 * @note While sending of events can be disabled by setting INVALID_EVENT_ID, consider disabling the interrupts at
 *       the source to save runtime.
 */
class TimerAdapter : public Timer::InterruptHandler
{
public:
	/**
	 * Constructor.
	 * @param receiver Receiver of the events.
	 * @param outputCompareMatchEvent %Event for Output Compare Match interrupts.
	 * @param overflowEvent %Event for Overflow interrupts.
	 */
	TimerAdapter(EventReceiver *receiver, const Event &outputCompareMatchEvent, const Event &overflowEvent) :
		m_receiver(receiver), m_outputCompareMatchEvent(outputCompareMatchEvent), m_overflowEvent(overflowEvent)
	{
	}

	/**
	 * Returns the currently set receiver.
	 * @result Currently set receiver.
	 */
	EventReceiver *getReceiver(void) const
	{
		return m_receiver;
	}

	/**
	 * Sets the event receiver.
	 * @param receiver Receiver to set.
	 */
	void setReceiver(EventReceiver *receiver)
	{
		m_receiver = receiver;
	}

	/**
	 * Returns the currently set event for Output Compare Match interrupts.
	 * @result Currently set event.
	 */
	Event getOutputCompareMatchEvent(void) const
	{
		return m_outputCompareMatchEvent;
	}

	/**
	 * Sets the event for Output Compare Match interrupts.
	 * @param event Event to set.
	 */
	void setOutputCompareMatchEvent(const Event &event)
	{
		m_outputCompareMatchEvent = event;
	}

	/**
	 * Returns the currently set event for Overflow interrupts.
	 * @result Currently set event.
	 */
	Event getOverflowEvent(void) const
	{
		return m_overflowEvent;
	}

	/**
	 * Sets the event for Overflow interrupts.
	 * @param event Event to set.
	 */
	void setOverflowEvent(const Event &event)
	{
		m_overflowEvent = event;
	}

	virtual void onOutputCompareMatch(void) override
	{
		if (m_receiver && m_outputCompareMatchEvent != Event::INVALID_EVENT_ID)
		{
			m_receiver->onEvent(m_outputCompareMatchEvent);
		}
	}

	virtual void onOverflow(void) override
	{
		if (m_receiver && m_overflowEvent != Event::INVALID_EVENT_ID)
		{
			m_receiver->onEvent(m_overflowEvent);
		}
	}

private:
	EventReceiver *m_receiver;
	Event m_outputCompareMatchEvent;
	Event m_overflowEvent;
};

#endif
