/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef TIMERSERVICE_H
#define TIMERSERVICE_H

/**
 * Base type for timer services that can be added to TimerServiceHost instances.
 * @internal Currently this class contains the running state, because all timer services happen
 *           to have it. If there should be timer services without this state, it should be
 *           moved to a new intermediate class.
 */
class TimerService
{
public:
	/**
	 * Constructor.
	 */
	TimerService(void) : m_state(STOPPED)
	{
	}

	/**
	 * Callback for base tick events of TimerServiceHost. Called from ISR context.
	 */
	virtual void onTick(void) = 0;

	/**
	 * Starts or continues the timer.
	 */
	void start(void)
	{
		m_state = RUNNING;
	}

	/**
	 * Stops the timer, keeping the current timeout value.
	 */
	void stop(void)
	{
		m_state = STOPPED;
	}

	/**
	 * Returns whether timer is currently running.
	 * @result True if timer is running, false otherwise.
	 */
	bool isRunning(void) const
	{
		return m_state == RUNNING;
	}

private:
	enum State
	{
		STOPPED,
		RUNNING,
	};

	State m_state;
};

#endif
