/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef USART_H
#define USART_H

#include <stdint.h>

/**
 * Interface for devices that implement an USART.
 * The interface also contains a callback data type for using interrupts, but this
 * is not required.
 */
class Usart
{
public:
	/**
	 * Callback type for use with interrupts.
	 */
	class InterruptHandler
	{
	public:
		/**
		 * Handler for Data Register Empty interrupt.
		 */
		virtual void onDataRegisterEmpty(void)
		{
		}

		/**
		 * Handler for RX Complete interrupt.
		 */
		virtual void onRxComplete(void)
		{
		}
	};

	/**
	 * Configuration for USART devices.
	 */
	struct Config
	{
		/**
		 * Setting for baud rate generation using the baud rate register and U2X mode.
		 */
		struct BaudRateSetting
		{
			/**
			 * Setting for baud rate register.
			 * @note This is NOT the baud rate! The actual baud rate also depends on the
			 *       CPU frequency, this register setting is just a divider of it.
			 */
			uint16_t baudRateRegister;
			/**
			 * Flag if speed doubling should be used (basically doubling the precision
			 * of the baud rate register for allowing finer tuning).
			 */
			bool doubleSpeed;
		};

		/**
		 * Number of data bits.
		 */
		enum DataBits
		{
			/// 5 data bits per frame.
			DATABITS_5,
			/// 6 data bits per frame.
			DATABITS_6,
			/// 7 data bits per frame.
			DATABITS_7,
			/// 8 data bits per frame.
			DATABITS_8,
		};

		/**
		 * Parity modes.
		 */
		enum Parity
		{
			/// No parity.
			PARITY_DISABLED,
			/// Odd parity bit.
			PARITY_ODD,
			/// Even parity bit.
			PARITY_EVEN,
		};

		/**
		 * Number of stop bits.
		 */
		enum StopBits
		{
			/// 1 stop bit per frame.
			ONE_STOPBIT,
			/// 2 stop bits per frame.
			TWO_STOPBITS,
		};

		/**
		 * Configuration of baud rate generator.
		 */
		BaudRateSetting baudRateSetting;
		/**
		 * Number of data bits.
		 */
		DataBits dataBits;
		/**
		 * Parity mode.
		 */
		Parity parity;
		/**
		 * Number of stop bits.
		 */
		StopBits stopBits;
	};

	/**
	 * Enables or disables the USART on hardware level.
	 * @param state Desired state of USART, true means enabled, false means disabled.
	 */
	virtual void enable(bool state) = 0;

	/**
	 * Configures the USART device.
	 * @param config Configuration for USART device.
	 */
	virtual void configure(const Config &config) = 0;

	/**
	 * Writes a data byte into the data register.
	 * @param data Data byte to write.
	 */
	virtual void setData(uint8_t data) = 0;

	/**
	 * Gets the content of the data register.
	 * @note This might be old or undefined data! Check with isReadable() first.
	 * @result Content of data register.
	 */
	virtual uint8_t getData(void) = 0;

	/**
	 * Checks if data is available for reading.
	 * @result True if data can be read, false otherwise.
	 */
	virtual bool isReadable(void) = 0;

	/**
	 * Checks if data register is ready for writing the next data byte.
	 * @result True if data can be written, false otherwise.
	 */
	virtual bool isWritable(void) = 0;

	/**
	 * Sets callback for interrupts.
	 * @param handler Pointer to class implementing the interrupt handler interface.
	 */
	virtual void setInterruptHandler(InterruptHandler *handler) = 0;

	/**
	 * Enables or disabled the Data Register Empty interrupt.
	 * @param state True to enable interrupt, false to disable.
	 */
	virtual void enableDataRegisterEmptyInterrupt(bool state) = 0;

	/**
	 * Enables or disabled the RX Complete interrupt.
	 * @param state True to enable interrupt, false to disable.
	 */
	virtual void enableRxCompleteInterrupt(bool state) = 0;

	/**
	 * Automatically fills a configuration structure for USART operation.
	 * @param config Configuration structure to fill.
	 * @param desiredBaudRate Desired baud rate, e.g. 9600.
	 * @param mode Mode string consisting of 3 characters: number of data bits, parity
	 *        (<b>o</b>dd/<b>e</b>ven/<b>n</b>one, case insensitive) and number of stop bits.
	 *        Example: "8n1".
	 * @param actualBaudRate Optional pointer to buffer to receive the baud rate that was
	 *        actually configured.
	 * @result True if configuration structure was filled successfully, false otherwise.
	 * @note Template parameter CPUFREQUENCY must contain the CPU frequency in Hertz.
	 * @note Using compile-time constants for baud rate reduces code size dramatically.
	 */
	template <uint32_t CPUFREQUENCY>
	static bool autoFill(Config &config, uint32_t desiredBaudRate, const char *mode, uint32_t *actualBaudRate = nullptr)
	{
		bool success = false;

		if (mode != nullptr && mode[0] != '\0' && mode[1] != '\0' && mode[2] != '\0')
		{
			success = insertBaudRate<CPUFREQUENCY>(config.baudRateSetting, desiredBaudRate, actualBaudRate) &&
				insertDataBits(config, mode[0]) && insertParity(config, mode[1]) && insertStopBits(config, mode[2]);
		}

		return success;
	}

private:
	template <class TYPE>
	static TYPE absDiff(TYPE a, TYPE b)
	{
		return (a > b) ? (a - b) : (b - a);
	}

	template <uint32_t CPUFREQUENCY>
	static bool insertBaudRate(Config::BaudRateSetting &setting, uint32_t desiredBaudRate, uint32_t *actualBaudRate = nullptr)
	{
		// Formulas from data sheet:
		//
		// Operating mode              | Calculation of Baud rate         | Calculation of UBRR value
		// ----------------------------+----------------------------------+-------------------------------
		// Normal mode (U2X = 0)       | BAUD = f_osc / (16 * (UBRR + 1)) | UBRR = f_osc / (16 * BAUD) - 1
		// Double speed mode (U2X = 1) | BAUD = f_osc / ( 8 * (UBRR + 1)) | UBRR = f_osc / ( 8 * BAUD) - 1

		// UBRR value for given baud rate
		const uint16_t registerValueNormal = 0.5 + CPUFREQUENCY  / (16.0 * desiredBaudRate) - 1;
		const uint16_t registerValueU2X    = 0.5 + CPUFREQUENCY  / ( 8.0 * desiredBaudRate) - 1;

		// Resulting baud rate from calculated UBRR to check amount of deviation
		const uint32_t resultingBaudRateNormal = 0.5 + CPUFREQUENCY / (16.0 * (registerValueNormal + 1));
		const uint32_t resultingBaudRateU2X    = 0.5 + CPUFREQUENCY / ( 8.0 * (registerValueU2X    + 1));

		// If double speed mode is more accurate, use it. Otherwise prefer normal mode.
		if (absDiff(resultingBaudRateU2X, desiredBaudRate) < absDiff(resultingBaudRateNormal, desiredBaudRate))
		{
			setting.baudRateRegister = registerValueU2X;
			setting.doubleSpeed = true;
			if (actualBaudRate != nullptr)
			{
				*actualBaudRate = resultingBaudRateU2X;
			}
		}
		else
		{
			setting.baudRateRegister = registerValueNormal;
			setting.doubleSpeed = false;
			if (actualBaudRate != nullptr)
			{
				*actualBaudRate = resultingBaudRateNormal;
			}
		}

		return true;
	}

	static bool insertDataBits(Config &config, char asciiDigit)
	{
		const Config::DataBits table[] = { Config::DATABITS_5, Config::DATABITS_6, Config::DATABITS_7, Config::DATABITS_8 };
		bool valid = false;

		if (asciiDigit >= '5' && asciiDigit <= '8')
		{
			config.dataBits = table[asciiDigit - '5'];
			valid = true;
		}

		return valid;
	}

	static bool insertParity(Config &config, char asciiLetter)
	{
		bool valid = false;

		switch (asciiLetter)
		{
		case 'n':
		case 'N':
			config.parity = Config::PARITY_DISABLED;
			valid = true;
			break;
		case 'o':
		case 'O':
			config.parity = Config::PARITY_ODD;
			valid = true;
			break;
		case 'e':
		case 'E':
			config.parity = Config::PARITY_EVEN;
			valid = true;
			break;
		}

		return valid;
	}

	static bool insertStopBits(Config &config, char asciiDigit)
	{
		bool valid = false;

		switch (asciiDigit)
		{
		case '1':
			config.stopBits = Config::ONE_STOPBIT;
			valid = true;
			break;
		case '2':
			config.stopBits = Config::TWO_STOPBITS;
			valid = true;
			break;
		}

		return valid;
	}
};

#endif
