/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef VIRTUALINPUTPORT_H
#define VIRTUALINPUTPORT_H

#include "inputport.h"
#include "inputpin.h"
#include "pinarray.h"

/**
 * Virtual 8 bit port consisting of 8 pins of type InputPin. This is a convenience wrapper for PinArray implementing
 * the InputPort interface.
 */
class VirtualInputPort : public InputPort
{
public:
	/**
	 * Constructor. Assigns pins to internal PinArray. The parameters intentionally don't have default values so
	 * that you have to make up your mind. See notes.
	 * @param pin0 Pin to be used for bit position 0.
	 * @param pin1 Pin to be used for bit position 1.
	 * @param pin2 Pin to be used for bit position 2.
	 * @param pin3 Pin to be used for bit position 3.
	 * @param pin4 Pin to be used for bit position 4.
	 * @param pin5 Pin to be used for bit position 5.
	 * @param pin6 Pin to be used for bit position 6.
	 * @param pin7 Pin to be used for bit position 7.
	 * @note You can specify null pointers for pins that should not be used (meaning bits that will read always 0).
	 * @note You can specify the same pin multiple times. But be aware of race conditions when reading the pin
	 *       state.
	 */
	VirtualInputPort(InputPin *pin0, InputPin *pin1, InputPin *pin2, InputPin *pin3, InputPin *pin4, InputPin *pin5, InputPin *pin6, InputPin *pin7)
	{
		m_pinArray.setPin(0, pin0);
		m_pinArray.setPin(1, pin1);
		m_pinArray.setPin(2, pin2);
		m_pinArray.setPin(3, pin3);
		m_pinArray.setPin(4, pin4);
		m_pinArray.setPin(5, pin5);
		m_pinArray.setPin(6, pin6);
		m_pinArray.setPin(7, pin7);
	}

	/**
	 * Get value being represented by set of pins.
	 * @result Value read from pins.
	 * @note Bits that are not associated with a pin read as 0.
	 */
	virtual uint8_t getValue(void) const override
	{
		return m_pinArray.getValue();
	}

private:
	PinArray<InputPin, uint8_t, 8> m_pinArray;
};

#endif
