/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef OUTPUTSHIFTREGISTERPORTWITHLATCH_H
#define OUTPUTSHIFTREGISTERPORTWITHLATCH_H

#include "outputshiftregisterport.h"
#include "outputshiftregisterwithlatch.h"

/**
 * Class that implements an OutputPort using a shift register featuring an output latch. It uses the
 * OutputShiftRegisterWithLatch interface that defines only a data, clock and output latch line, thus there will be
 * always 8 bits transported (instead of clearing the register and making use of leading zeros).
 * See OutputShiftRegisterPort for a description of the autoFlush property.
 * Additionally OutputShiftRegisterPortWithLatch defines a autoLatch property that works similar: if set to true, the
 * output latch will be triggered automatically with each data transfer to the device (either bei setValue() or by
 * flush()). Otherwise an explicit call to latch() is required to make the new data appear at the device's outputs.
 */
class OutputShiftRegisterPortWithLatch : public OutputShiftRegisterPort
{
public:
	/**
	 * Constructor.
	 * @param shiftRegister Pointer to OutputShiftRegister instance to be used as output device.
	 * @param autoFlush Value of autoFlush property, defaulting to true.
	 * @param autoLatch Value of autoLatch property, defaulting to true.
	 */
	OutputShiftRegisterPortWithLatch(OutputShiftRegisterWithLatch *shiftRegister, bool autoFlush = true, bool autoLatch = true) :
		OutputShiftRegisterPort(shiftRegister, autoFlush), m_autoLatch(autoLatch)
	{
	}

	/**
	 * Getter for current value of autoLatch property.
	 * @result Current value of autoLatch property.
	 */
	bool getAutoLatch(void) const
	{
		return m_autoLatch;
	}

	/**
	 * Setter for value of autoLatch property.
	 * @param state New value for autoLatch property, true means that autoLatch is active.
	 */
	void setAutoLatch(bool state)
	{
		m_autoLatch = state;
	}

	/**
	 * Excplicitly transferrs internal value to the associated device.
	 */
	void flush(void)
	{
		OutputShiftRegisterPort::flush();

		if (m_autoLatch)
		{
			latch();
		}
	}

	/**
	 * Excplicitly triggers output latch of the associated device.
	 */
	void latch(void)
	{
		static_cast<OutputShiftRegisterWithLatch*>(m_shiftRegister)->latchOutput();
	}

private:
	bool m_autoLatch;
};

#endif
