/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef SHIFTREGISTER74HC164_H
#define SHIFTREGISTER74HC164_H

#include "integratedcircuit.h"
#include "outputshiftregister.h"
#include "outputpin.h"

/**
 * Implementation of the 74HC164 serial in/parallel out shift register with two data inputs, clock and clear function.
 * Each input of the hardware device can be connected using a OutputPin instance. For functions that are not required
 * it is safe to pass a null pointer.
 *
 * Pinout of device:
 * @code
 *        +--v--+
 *   A  1 |     | 14  VCC
 *   B  2 |     | 13  QH
 *  QA  3 |     | 12  QG
 *  QB  4 |     | 11  QF
 *  QC  5 |     | 10  QE
 *  QD  6 |     |  9  /CLR
 * GND  7 |     |  8  CLK
 *        +-----+
 * @endcode
 *
 * - A and B are connected via logic AND
 * - /CLR clears when being held LOW
 * - CLK triggers on rising edge
 */
class ShiftRegister74HC164 : public OutputShiftRegister,
	public IntegratedCircuit::hasClear
{
public:
	/**
	 * Constructor. Pins that are not used can be passed as null pointer.
	 * @param pinDataA Pin to be used for Data A input.
	 * @param pinDataB Pin to be used for Data B input.
	 * @param pinClock Pin to be used for clocking data into device.
	 * @param pinClear Pin to be used to clear device.
	 */
	ShiftRegister74HC164(OutputPin *pinDataA, OutputPin *pinDataB, OutputPin *pinClock, OutputPin *pinClear) :
		m_dataA(pinDataA), m_dataB(pinDataB), m_clock(pinClock), m_clear(pinClear)
	{
	}

	virtual void setData(bool value) override
	{
		setDataA(value);
		setDataB(value);
	}

	/**
	 * Sets data A input to specified value.
	 * @param value Value of data input, true for HIGH level and false for LOW level.
	 */
	void setDataA(bool value)
	{
		if (m_dataA)
		{
			m_dataA->setValue(value);
		}
	}

	/**
	 * Sets data B input to specified value.
	 * @param value Value of data input, true for HIGH level and false for LOW level.
	 */
	void setDataB(bool value)
	{
		if (m_dataB)
		{
			m_dataB->setValue(value);
		}
	}

	virtual void clock(void) override
	{
		if (m_clock)
		{
			m_clock->setValue(false);
			m_clock->setValue(true);
		}
	}

	virtual void clear(void) override
	{
		if (m_clear)
		{
			m_clear->setValue(false);
			m_clear->setValue(true);
		}
	}

private:
	OutputPin * const m_dataA;
	OutputPin * const m_dataB;
	OutputPin * const m_clock;
	OutputPin * const m_clear;
};

#endif
