/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef SHIFTREGISTER74HC595_H
#define SHIFTREGISTER74HC595_H

#include "integratedcircuit.h"
#include "outputshiftregisterwithlatch.h"
#include "outputpin.h"

/**
 * Implementation of the 74HC595 serial in/parallel out shift register with output latch and output enable functions.
 * Each input of the hardware device can be connected using a OutputPin instance. For functions that are not required
 * it is safe to pass a null pointer.
 *
 * Pinout of device:
 * @code
 *        +--v--+
 *  QB  1 |     | 16  VCC
 *  QC  2 |     | 15  QA
 *  QD  3 |     | 14  SER
 *  QE  4 |     | 13  /OE
 *  QF  5 |     | 12  RCLK
 *  QG  6 |     | 11  SRCLK
 *  QH  7 |     | 10  /SRCLR
 * GND  8 |     |  9  QH'
 *        +-----+
 * @endcode
 *
 * - SER is serial input of bits
 * - /SRCLR clears shift register when being held LOW
 * - SRCLK is shift register clock, triggers on rising edge
 * - RCLK is latch clock, triggers on rising edge
 * - /OE enables outputs when LOW, otherwise outputs are disabled (3-state)
 */
class ShiftRegister74HC595 : public OutputShiftRegisterWithLatch,
	public IntegratedCircuit::hasClear,
	public IntegratedCircuit::hasOutputEnable
{
public:
	/**
	 * Constructor. Pins that are not used can be passed as null pointer.
	 * @param pinData Pin to be used for data input.
	 * @param pinClock Pin to be used for clocking data into device.
	 * @param pinClear Pin to be used to clear device.
	 * @param pinLatch Pin to be used to trigger latch.
	 * @param pinEnable Pin to be used to enable/disable the device's outputs.
	 */
	ShiftRegister74HC595(OutputPin *pinData, OutputPin *pinClock, OutputPin *pinClear, OutputPin *pinLatch, OutputPin *pinEnable) :
		m_data(pinData), m_clock(pinClock), m_clear(pinClear), m_latch(pinLatch), m_enable(pinEnable)
	{
	}

	void setData(bool value)
	{
		if (m_data)
		{
			m_data->setValue(value);
		}
	}

	virtual void clock(void) override
	{
		if (m_clock)
		{
			m_clock->setValue(false);
			m_clock->setValue(true);
		}
	}

	virtual void clear(void) override
	{
		if (m_clear)
		{
			m_clear->setValue(false);
			m_clear->setValue(true);
		}
	}

	virtual void enableOutput(bool enable) override
	{
		if (m_enable)
		{
			m_enable->setValue(!enable);
		}
	}

	virtual void latchOutput(void) override
	{
		if (m_latch)
		{
			m_latch->setValue(false);
			m_latch->setValue(true);
		}
	}

private:
	OutputPin * const m_data;
	OutputPin * const m_clock;
	OutputPin * const m_clear;
	OutputPin * const m_latch;
	OutputPin * const m_enable;
};

#endif
